@@ -9,21 +9,19 @@
 
 ## Configuration
 # Commands
-$WGET = "/usr/bin/wget";
 $LYNX = "/usr/bin/lynx";
-#$SED = "/bin/sed";
-$GREP = "/bin/grep";
+#$SED = "/usr/bin/sed";
+$GREP = "/usr/bin/grep";
 $RECODE = "NO"; #"/usr/bin/recode"; -- dont use recode, it is not widely available
 # Directories
-$prefix="/home/cs/onlinebanking";
+$BANKDATADIR="/home/stimming/usr/share/ktoblzcheck";
+$prefix="/home/stimming/usr";
 $datadir="${prefix}/share";
 $pkgdatadir="$datadir/ktoblzcheck";
-$BANKDATADIR="${pkgdatadir}";
 
 ## Sanity checks
-die "Neither wget nor lynx is available on your system, or was available on the system where the installed rpm was built. This tool requires wget or lynx. If you have it i
-nstalled, then edit the script and set the variable WGET or LYNX to the full path to wget or lynx. Aborting for now."
-  if $WGET eq "NO" and $LYNX eq "NO";
+die "lynx is not available on your system, or was not available on the system where the installed rpm was built. This tool requires lynx. If you have it installed, then edit the script and set the variable LYNX to the full path to lynx. Aborting for now."
+  if $LYNX eq "NO";
 #die "sed or grep is not available on your system. This tool requires it. Aborting."
 #  if $SED eq "NO" || $GREP eq "NO";
 die "The directory for the bankdata \"$BANKDATADIR\" does not exist. Aborting."
@@ -36,30 +34,22 @@ die "The directory for the bankdata \"$BANKDATADIR\" is not writable. You need t
 ## Common constants
 $debug=0; # set this to nonzero for activating debugging mode
 $BASE_URL="http://www.bundesbank.de";
-$MAIN_URL="$BASE_URL";
-$MAIN_DOC="$MAIN_URL/Redaktion/DE/Standardartikel/Kerngeschaeftsfelder/Unbarer_Zahlungsverkehr/bankleitzahlen_download.html";
+$MAIN_URL="$BASE_URL/zahlungsverkehr";
+$MAIN_DOC="$MAIN_URL/zahlungsverkehr_bankleitzahlen_download.php";
+$INSTALLED_NAME="bankdata.txt";
 
 if ($debug == 0) {
-  if ($WGET ne "NO") {
-    $DOWNLOADCMD = "$WGET -O - $MAIN_DOC";
-  } else {
-    $DOWNLOADCMD = "$LYNX -source $MAIN_DOC";
-  }
+  $LYNXCMD = "$LYNX -source $MAIN_DOC";
 } else {
   print "DEBUG MODE! Will only echo commands, not execute them.\n";
-  $DOWNLOADCMD = "cat ~/privat/hbci/blabla.html";
+  $LYNXCMD = "cat ~/privat/hbci/blabla.html";
 }
-# Old regexp before 2006-06-04
-#$datafile_regexp = 'blz\d{4,8}?pc\.txt';
-# Regexp from 2006-06-05 onwards
-#$datafile_regexp = 'blz_\d{8}\.txt';
-# New regexp from 2012-05-30 onwards
-$datafile_regexp = 'blz_\d{4}_\d{2}_\d{2}_txt\.txt';
+$datafile_regexp = 'blz\d{4,8}?pc\.txt';
 
 ## Find the available potential download files
 @potential_files = ();
 @potential_urls = ();
-open(INPUT, "$DOWNLOADCMD |") || die "can't open $DOWNLOADCMD: $!";;
+open(INPUT, "$LYNXCMD |") || die "can't open $LYNXCMD: $!";;
 while (<INPUT>) {
   my ($addr) = m|($datafile_regexp)|giosx;
   if ($addr) {
@@ -99,8 +89,8 @@ if (scalar @diff == 0) {
   $k = 0;
   foreach $e (@diff) { print " [".$k++."] ".$e."\n"; };
   print "Which one should be chosen? (Hint: The file names contain \n".
-    "the four-digit year, the month and the date as YYYY_MM_DD. You should \n".
-      "probably choose the file that corresponds to a date that \n".
+    "the two-digit year and month as YYMM. You should probably \n".
+      "choose the file that corresponds to a year/month pair that \n".
 	"lies in the future.) \n".
 	  "Please type the corresponding number and press Enter.\n";
   $key = <STDIN>;
@@ -112,12 +102,6 @@ if (scalar @diff == 0) {
 ($correct_url_line) = grep (m|$file|, @potential_urls);
 ($correct_url) = ($correct_url_line =~ m|([^"]*$file[^"]*)|gio);
 
-# The installation name is now taken from the original filename;
-# "blz_20110102" will be installed after conversion as
-# "bankdata_20110102".
-$INSTALLED_NAME = $file;
-$INSTALLED_NAME =~ s/blz_(\d{4})_(\d{2})_(\d{2})_txt\.txt/bankdata_\1\2\3.txt/gio;
-
 $download = "$MAIN_URL/$correct_url";
 $install_1 = "$BANKDATADIR/$file";
 $install_2 = "$BANKDATADIR/$INSTALLED_NAME";
@@ -144,13 +128,8 @@ if ($debug == 0) {
 ## Now the real work:
 ##
 ## 1. Download
-if ($WGET ne "NO") {
-  print "$ECHO $WGET -O $install_1 $download\n";
-  system("$ECHO $WGET -O $install_1 $download");
-} else {
-  print "$ECHO $LYNX -source $download $gt $install_1\n";
-  system("$ECHO $LYNX -source $download $gt $install_1");# || die "can't call system: $!";
-}
+print "$ECHO $LYNX -source $download $gt $install_1\n";
+system("$ECHO $LYNX -source $download $gt $install_1");# || die "can't call system: $!";
 
 ## 2. Check whether the Bundesbank's BLZ is in there
 print "$GREP -q 10000000 $install_1\n";
@@ -158,10 +137,9 @@ if (system("$GREP -q 10000000 $install_1") != 0) {
   die "Downloaded file $install_1 is not in valid BLZ format. Aborting.";
 }
 
-### 3. Backup copy of old bankdata file
-#print "$ECHO cp $install_2 $install_2~\n";
-#system("$ECHO cp $install_2 $install_2~");# || die "can't call system: $!";
-# - no backup needed anymore because all files have different dates in the name
+## 3. Backup copy of old bankdata file
+print "$ECHO cp $install_2 $install_2~\n";
+system("$ECHO cp $install_2 $install_2~");# || die "can't call system: $!";
 
 ## 4. Recode the downloaded file from its latin1 encoding into the
 ## locale encoding
