@@ -1,6 +1,6 @@
 /**-*-c++-*-****************************************************************
                              -------------------
-    cvs         : $Id: ktoblzcheck.h,v 1.7 2004/09/10 08:25:27 cstim Exp $
+    cvs         : $Id: ktoblzcheck.h,v 1.13 2005/07/29 18:31:32 cstim Exp $
     begin       : Sat Aug 10 2002
     copyright   : (C) 2002, 2003 by Fabian Kaiser and Christian Stimming
     email       : fabian@openhbci.de
@@ -27,15 +27,23 @@
 #ifndef KTOBLZCHECK_H
 #define KTOBLZCHECK_H
 
+/** @file 
+ * @brief Checking of German BLZ and account numbers
+ *
+ * This file contains the classes and C wrappers for checking German
+ * BLZ (bank codes) and account numbers.
+ */
 
 #ifdef __cplusplus
 
 #include <string>
 #include <map>
 
-/** Class that stores a list of known banks, returns banks with given
- * bank codes and validates account numbers accordings to the bank's
- * known validation/checking algorithms.
+/** @brief German bank information database and account code verification
+ *
+ * Class that stores a list of known German banks, returns banks with
+ * given bank codes, and validates account numbers accordings to the
+ * bank's known validation/checking algorithms.
  *
  * The list of known banks is read from the configuration file when
  * this object is constructed.
@@ -72,7 +80,8 @@ public:
       BANK_NOT_KNOWN = 3
   } Result;
   
-  /**
+  /** @brief Available information about one bank
+   *
    * This class holds information about one bank.
    *
    * <ul>
@@ -108,6 +117,10 @@ public:
     Record(unsigned long id, const std::string& method, 
 	   const std::string& name, 
 	   const std::string& loc);
+    /** Constructor with all values from strings */
+    Record(const char *id, const char *method, 
+	   const char *name, 
+	   const char *loc);
   };
 
   /**
@@ -188,17 +201,55 @@ public:
    */
   void createIndex();
 
+  /**
+   * Returns a meaningful english string explaining the result value
+   */
+  static std::string resultToString(Result r);
+
+  /** @name Ktoblzcheck library information */
+  //@{
+  /** 
+   * Returns the character encoding that is used when strings are
+   * returned. So far this has been "ISO-8859-15" but at some point in
+   * the future it might change into "UTF-8".
+   */
+  static const char* stringEncoding();
+  /**
+   * Returns the value of ktoblzcheck's configuration variable
+   * VERSION, which can be "1.6" or something similar. 
+   */
+  static const char* libraryVersion();
+  //@}
 
 private:
-  /* The list of the bank data */
+  /** The type of the list of the bank data */
   typedef std::map<unsigned long, Record*> banklist_type;
+  /** The list of the bank data */
   banklist_type data;
 
+public:
+#ifndef DOXYGEN_IGNORE
+  /** Internal callback function */
+  typedef Result (*MethodFunc)(int *account, int *weight);
+  /** Internal callback function */
+  typedef Result (*MethodFuncLong)(int *account, int *weight, 
+				   const std::string& accountId, const std::string& bankId);
+#endif // DOXYGEN_IGNORE
+
+private:
+  typedef std::map<std::string, MethodFunc> method_map_t;
+  method_map_t method_map;
+  typedef std::map<std::string, MethodFuncLong> method_map2_t;
+  method_map2_t method_map2;
+
   /** Deletes all records inside the bank data list */
   void deleteList();
   /** The function to actually read data from file into the list;
       clears all existing data. */
   void readFile(const std::string &filename);
+
+  /** Initialized the map of method string to the respective callback function */
+  void initMethodMap();
 };
 
 typedef AccountNumberCheck::Result AccountNumberCheck_Result;
@@ -211,6 +262,7 @@ typedef struct AccountNumberCheck AccountNumberCheck;
 typedef struct AccountNumberCheck_Record AccountNumberCheck_Record;
 #endif /* __cplusplus */
 
+  /** @name AccountNumberCheck methods */
   /*@{*/
 
   /**
@@ -264,9 +316,9 @@ typedef struct AccountNumberCheck_Record AccountNumberCheck_Record;
    * <li><b>3</b> No bank with the specified bankid could be found
    * </ul>
    *
+   * @param a The AccountNumberCheck object
    * @param bankId The bank code (BLZ) of the bank to test
    * @param accountId The account id to check
-   * @param method If set, force the use of specified check-method
    */
   extern AccountNumberCheck_Result
   AccountNumberCheck_check(const AccountNumberCheck *a, 
@@ -303,6 +355,22 @@ typedef struct AccountNumberCheck_Record AccountNumberCheck_Record;
   extern void AccountNumberCheck_createIndex(AccountNumberCheck *a);
   /*@}*/
 
+  /** @name Ktoblzcheck library information */
+  /*@{*/
+  /** 
+   * Returns the character encoding that is used when strings are
+   * returned. So far this has been "ISO-8859-15" but at some point in
+   * the future it might change into "UTF-8".
+   */
+  extern const char* AccountNumberCheck_stringEncoding();
+  /**
+   * Returns the value of ktoblzcheck's configuration variable
+   * VERSION, which can be "1.6" or something similar. 
+   */
+  extern const char* AccountNumberCheck_libraryVersion();
+  /*@}*/
+
+  /** @name AccountNumberCheck_Record methods */
   /*@{*/
   /** Destructor */
   extern void
