@@ -1,6 +1,6 @@
 /***************************************************************************
                              -------------------
-    cvs         : $Id: algorithms.cc,v 1.7 2004/09/08 12:56:03 cstim Exp $
+    cvs         : $Id: algorithms.cc,v 1.10 2005/01/20 12:57:48 cstim Exp $
     begin       : Sat Aug 10 2002
     copyright   : (C) 2002 by Fabian Kaiser
     email       : fabian@openhbci.de
@@ -30,6 +30,7 @@
 #include "algorithms.h"
 #include <fstream>
 #include <iostream>
+#include <assert.h>
 
 
 #if OS_WIN32
@@ -40,6 +41,90 @@
 
 using namespace std;
 
+
+
+/* Divisions-Makros (copied from Michael Plugge
+   <m.plugge@fh-mannheim.de> and his konto_check package. */
+
+/* Makros zur Modulo-Bildung ber iterierte Subtraktionen.
+ * auf Intel-Hardware ist dies schneller als eine direkte Modulo-Operation;
+ * Auf Alpha ist Modulo allerdings schneller (gute FPU).
+ */
+#ifdef __ALPHA
+#   define SUB_352(a)   (a)%=11
+#   define SUB_176(a)   (a)%=11
+#   define SUB_88(a)   (a)%=11
+#   define SUB_44(a)   (a)%=11
+#   define SUB_22(a)   (a)%=11
+#   define SUB_11(a)   (a)%=11
+
+#   define SUB_320(a)   (a)%=10
+#   define SUB_160(a)   (a)%=10
+#   define SUB_80(a)   (a)%=10
+#   define SUB_40(a)   (a)%=10
+#   define SUB_20(a)   (a)%=10
+#   define SUB_10(a)   (a)%=10
+
+#   define SUB_288(a)   (a)%=9
+#   define SUB_144(a)   (a)%=9
+#   define SUB_72(a)   (a)%=9
+#   define SUB_36(a)   (a)%=9
+#   define SUB_18(a)   (a)%=9
+#   define SUB_9(a)   (a)%=9
+
+#   define SUB_224(a)   (a)%=7
+#   define SUB_112(a)   (a)%=7
+#   define SUB_56(a)   (a)%=7
+#   define SUB_28(a)   (a)%=7
+#   define SUB_14(a)   (a)%=7
+#   define SUB_7(a)   (a)%=7
+#else
+#   define SUB(a,x) if((a)>=(x)) (a)-=(x)
+#   define SUB_352(a) SUB((a),352); SUB_176(a)
+#   define SUB_176(a) SUB((a),176); SUB_88(a)
+#   define SUB_88(a)  SUB((a),88);  SUB_44(a)
+#   define SUB_44(a)  SUB((a),44);  SUB_22(a)
+#   define SUB_22(a)  SUB((a),22);  SUB_11(a)
+#   define SUB_11(a)  SUB((a),11)
+
+#   define SUB_320(a) SUB((a),320); SUB_160(a)
+#   define SUB_160(a) SUB((a),160); SUB_80(a)
+#   define SUB_80(a)  SUB((a),80);  SUB_40(a)
+#   define SUB_40(a)  SUB((a),40);  SUB_20(a)
+#   define SUB_20(a)  SUB((a),20);  SUB_10(a)
+#   define SUB_10(a)  SUB((a),10)
+
+#   define SUB_288(a) SUB((a),288); SUB_144(a)
+#   define SUB_144(a) SUB((a),144); SUB_72(a)
+#   define SUB_72(a)  SUB((a),72); SUB_36(a)
+#   define SUB_36(a)  SUB((a),36); SUB_18(a)
+#   define SUB_18(a)  SUB((a),18); SUB_9(a)
+#   define SUB_9(a)   SUB((a),9)
+
+#   define SUB_224(a) SUB((a),224); SUB_112(a)
+#   define SUB_112(a) SUB((a),112); SUB_56(a)
+#   define SUB_56(a)  SUB((a),56);  SUB_28(a)
+#   define SUB_28(a)  SUB((a),28);  SUB_14(a)
+#   define SUB_14(a)  SUB((a),14);  SUB_7(a)
+#   define SUB_7(a)   SUB((a),7)
+#endif
+
+// Alternative macros
+#ifdef __ALPHA
+#else
+#define SUBIP(a,x)  (a>=x ? a-x : a)
+#define SUBIP_10(a) SUBIP(a,10)
+#define SUBIP_20(a) SUBIP_10(SUBIP(a,20))
+#define SUBIP_40(a) SUBIP_20(SUBIP(a,40))
+#define SUBIP_80(a) SUBIP_40(SUBIP(a,80))
+#endif
+// However, it is difficult to evaluate the performance improvement
+// since especially valgrind doesn't show the extra cost of the modulo
+// operation. These macros are therefore not used so far.
+
+
+// ////////////////////
+
 std::string accnum_getRegKey(const char *value_name) 
 {
 #if OS_WIN32
@@ -96,6 +181,7 @@ std::string accnum_getRegKey(const char *value_name)
 #endif // OS_WIN32
 }
 
+// ////////////////////
 
 AccountNumberCheck::Result
 algo01(int modulus, int weight[10], bool crossfoot, 
@@ -134,7 +220,7 @@ int algo03(int modulus, int weight[10], bool crossfoot, int accountId[10],
 
   // for 00,...: calc the crossfoot for each value
   if (crossfoot)
-	crossFoot(res, res, 0, 9);
+	crossFoot(res);
 
   // add all values
   int result = add(res, startAdd, stopAdd);
@@ -153,7 +239,7 @@ int algo03a(int weight[10], bool crossfoot, int accountId[10],
 
   // for 00,...: calc the crossfoot for each value
   if (crossfoot)
-	crossFoot(res, res, 0, 9);
+	crossFoot(res);
 
   // add all values
   int result = add(res, startAdd, stopAdd);
@@ -162,7 +248,7 @@ int algo03a(int weight[10], bool crossfoot, int accountId[10],
 }
 
 AccountNumberCheck::Result
-algo05(string accountId) {
+algo05(const std::string& accountId) {
   int account[10];
   int weight[10];
   
@@ -200,7 +286,7 @@ algo05(string accountId) {
 	  res[i] = (res[i] + weight[i]) % 11;
 
 	// sum the stuff
-	int result = add(res, 0, 9) % 10;
+	int result = add_10(res) % 10;
 	
 	if (result == account[9])
 	  return AccountNumberCheck::OK;
@@ -209,7 +295,7 @@ algo05(string accountId) {
 }
  
 AccountNumberCheck::Result
-algo06(string accountId) {
+algo06(const std::string& accountId) {
   int i = 0;
   int c2 = 0;
   int d2 = 0;
@@ -291,30 +377,32 @@ algo06(string accountId) {
   }
   
   while( a5 < 0 || a5 > 4) {
-	if (a5 > 4) {
+      if (a5 > 4) {
 	  a5 -= 5;
-	} else {
+      } else {
 	  a5 += 5;
-	}
-	if (0 == d2) {
-	  p = tab1[a5];
-	} else {
-	  p = tab2[a5];
-	}
+      }
+  }
+  // Fixed wrong ending of while-loop; by Erik Kerger <erik@kedo.com>
+
+  if (0 == d2) {
+      p = tab1[a5];
+  } else {
+      p = tab2[a5];
   }
 
   if (p == account[9]) {
-	return AccountNumberCheck::OK;
+      return AccountNumberCheck::OK;
   } else {
-	if (0 == account[3]) {
+      if (0 == account[3]) {
 	  if (p > 4) {
-		p -= 5;
+	      p -= 5;
 	  } else {
-		p += 5;
+	      p += 5;
 	  }
 	  if (p == account[9])
-		return AccountNumberCheck::OK;
-	}	  
+	      return AccountNumberCheck::OK;
+      }	  
   }
    
   // hugh...
@@ -323,7 +411,7 @@ algo06(string accountId) {
 }
    
 AccountNumberCheck::Result 
-algo04(string bankId, string accountId) {
+algo04(const std::string& bankId, std::string accountId) {
 
   while (8 < accountId.length() && '0' == accountId[0])
     accountId = accountId.substr(1);
@@ -371,8 +459,7 @@ algo04(string bankId, string accountId) {
   multArray(eser10, weight10, res10);
 
   // add-mod
-  int result = (add(res02, 0, 9));
-  result += (add(res10, 0, 9));
+  int result = add_10(res02) + add_10(res10);
   result = result % 11;
 
   // find a multiple of weight[checkNumberIndex] that fits
@@ -398,7 +485,7 @@ algo04(string bankId, string accountId) {
 // same as algo04, but for nine digit accountIDs with
 // a little different eser12 system.
 AccountNumberCheck::Result 
-algo04a(string bankId, string accountId) {
+algo04a(const std::string& bankId, std::string accountId) {
 
   while (9 < accountId.length() && '0' == accountId[0])
     accountId = accountId.substr(1);
@@ -456,10 +543,10 @@ algo04a(string bankId, string accountId) {
   multArray(eser02, weight02, res02);
   multArray(eser10, weight10, res10);
 
-  int result = (add(res02, 0, 9));
+  int result = add_10(res02);
   // cout << "result add1: " << result << endl;
   // cout << "result add2: " << add(res10, 0, 9) << endl;
-  result += (add(res10, 0, 9));
+  result += add_10(res10);
   // cout << "result: " << result << endl;
   result = result % 11;
   // cout << "result: " << result << endl;
@@ -492,57 +579,119 @@ algo07(int accountId[10], int transform[6][10]) {
   return AccountNumberCheck::ERROR;
 }
 
-void multArray(int a[10], int b[10], int dest[10]) {
-  for (int i=0; i<10; i++) {
-	dest[i] = a[i] * b[i];
+void multArray(const int *a, const int *b, int dest[10]) {
+#define MULTARRAY(i) dest[i] = a[i] * b[i]
+    MULTARRAY(0);
+    MULTARRAY(1);
+    MULTARRAY(2);
+    MULTARRAY(3);
+    MULTARRAY(4);
+    MULTARRAY(5);
+    MULTARRAY(6);
+    MULTARRAY(7);
+    MULTARRAY(8);
+    MULTARRAY(9);
+    // Sigh -- writing it as above greatly speeds up runtime compared
+    // to a conventional for-loop:
+//     for (int i=0; i<10; i++) {
+// 	dest[i] = a[i] * b[i];
+//     }
+}
+
+// This one is unused
+void crossFoot(const int *source, int dest[10], int start, int stop) {
+    for (int i=start; i<stop+1; i++) {
+	int elem = source[i];
+	dest[i] = elem / 10  +  elem % 10;
   }
 }
 
-void crossFoot(int source[10], int dest[10], int start, int stop) {
-  for (int i=start; i<stop+1; i++) {
-	int t = source[i] / 10;
-	int o = source[i] % 10;
 
-	dest[i] = t + o;
-  }
+void crossFoot(int *source) {
+#define CROSSFOOTOP(i)  source[i] = source[i] / 10 + source[i] % 10
+    CROSSFOOTOP(0);
+    CROSSFOOTOP(1);
+    CROSSFOOTOP(2);
+    CROSSFOOTOP(3);
+    CROSSFOOTOP(4);
+    CROSSFOOTOP(5);
+    CROSSFOOTOP(6);
+    CROSSFOOTOP(7);
+    CROSSFOOTOP(8);
+    CROSSFOOTOP(9);
 }
 
-int add(int source[10], int start, int stop) {
-  int result = 0;
-  for (int i=start; i<stop+1; i++) {
-	result += source[i];
-  }
-
-  return result;
+int add(const int *source, int start, int stop) {
+    const int *sp = source + start;
+    switch (stop-start) {
+    case 0:
+	return sp[0];
+    case 1:
+	return sp[0] + sp[1];
+    case 2:
+	return sp[0] + sp[1] + sp[2];
+    case 3:
+	return sp[0] + sp[1] + sp[2] + sp[3];
+    case 4:
+	return sp[0] + sp[1] + sp[2] + sp[3] + sp[4];
+    case 5:
+	return sp[0] + sp[1] + sp[2] + sp[3] + sp[4] 
+	    + sp[5];
+    case 6:
+	return sp[0] + sp[1] + sp[2] + sp[3] + sp[4] 
+	    + sp[5] + sp[6];
+    case 7:
+	return sp[0] + sp[1] + sp[2] + sp[3] + sp[4] 
+	    + sp[5] + sp[6] + sp[7];
+    case 8:
+	return sp[0] + sp[1] + sp[2] + sp[3] + sp[4] 
+	    + sp[5] + sp[6] + sp[7] + sp[8];
+    case 9:
+	return sp[0] + sp[1] + sp[2] + sp[3] + sp[4] 
+	    + sp[5] + sp[6] + sp[7] + sp[8] + sp[9];
+    default:
+	assert(0);
+    }
 }
 
 string array2Number(int a[10]) {
-  string result;
-  char c;
+    string result("0000000000");
+    for (unsigned i=0; i<10; i++) {
+	result[i] = '0' + a[i];
+    }
 
-  for (int i=0; i<10; i++) {
-	c = 48 + a[i];
-	result += c;
-  }
-
-  return result;
+    return result;
 }
 
-void number2Array(string number, int a[10]) {
-  while (number.length() < 10)
-	number = "0" + number;
-	
-  for (int i=9; i>-1; i--) {
-	a[i] = number[i] - 48;
-  }
+void number2Array(const string& number, int a[10]) {
+    if (number.size() == 10) {
+#define ASSIGNARRAY(i)  a[i] = number[i] - '0'
+	ASSIGNARRAY(0);
+	ASSIGNARRAY(1);
+	ASSIGNARRAY(2);
+	ASSIGNARRAY(3);
+	ASSIGNARRAY(4);
+	ASSIGNARRAY(5);
+	ASSIGNARRAY(6);
+	ASSIGNARRAY(7);
+	ASSIGNARRAY(8);
+	ASSIGNARRAY(9);
+    } else {
+	unsigned padd_size = 10 - number.size();
+	unsigned i = 0;
+	for ( ; i < padd_size; i++)
+	    a[i] = 0;
+	for ( ; i < 10; i++) 
+	    a[i] = number[i-padd_size] - '0';
+    }
 }
 
-long_long number2LongLong(string number) {
+long_long number2LongLong(const string& number) {
   long_long result = 0;
 
   for (unsigned int i=0; i<number.length(); i++) {
 	result *= 10;
-	result += number[i] - 48;
+	result += number[i] - '0';
   }
 
   return result;
